function tx_out = tx_burst(data_psdu, mode)
%TX_BURST Summary of this function goes here
%   Detailed explanation goes here

% system parameters
% Signal symbol encoding
N_fft = 64;
N_gi = 16;
N_carriers = 52;
N_pilots = 4;
P_pilots = [-21 -7 7 21];
P_data = [-26:-22 -20:-8 -6:-1 1:6 8:20 22:26];
P_data_mod = mod(P_data+N_fft, N_fft) + 1;
P_pilots_mod = mod(P_pilots+N_fft, N_fft) + 1;
L_long = [1 1 -1 -1 1 1 -1 1 -1 1 1 1 1 1 1 -1 -1 1 1 -1 1 -1 1 1 1 1 0 ...
       1 -1 -1 1 1 -1 1 -1 1 -1 -1 -1 -1 -1 1 1 -1 -1 1 -1 1 -1 1 1 1 1];

N_RATE_FIELD = 4;
N_RESERVED_FIELD = 1;
N_TX_LENGTH_FIELD = 12;
N_PARITY_FIELD = 1;
N_TAIL_FIELD = 6;
N_SERVICE_FIELD = 16;
N_SCRAMBLER_STATES = 7;

trellis = poly2trellis(7,[133 171]);

%tx_len = 900; % 1 - 4095
%mode = 54; % 6, 9, 12, 18, 24, 36, 48, 54
%data_psdu = randi([0 1], 1, tx_len*8);
tx_len = length(data_psdu)/8; 

if mod(length(data_psdu),8) ~= 0,
   error('PSDU length should be multiple of 8\n'); 
end


% Generate long training sequence
P_long = mod((-26:26)+N_fft, N_fft) + 1;
tx_freq = zeros(1,N_fft);
tx_freq(P_long) = L_long;
tx_long = ifft(tx_freq);
tx_long = [tx_long(end-2*N_gi+1:end) tx_long tx_long tx_long(1)]; % one sample overlap

tx_long(1) = tx_long(1)*0.5;
tx_long(end) = tx_long(end)*0.5;


% Encode signal field
if mode==6,
    rate = [1 1 0 1];
    N_cbps = 48;
    N_dbps = 24;
    N_bpsc = 1;
    cr = 1/2;      
    P_pattern = [1 1];
      
elseif mode==9,
    rate = [1 1 1 1];
    N_cbps = 48;
    N_dbps = 36;
    N_bpsc = 1;
    cr = 3/4; 
    P_pattern = [1 1 1 0 0 1 1 1 1 0 0 1 1 1 1 0 0 1];      
    
elseif mode==12,
    rate = [0 1 0 1];
    N_cbps = 96;
    N_dbps = 48; 
    N_bpsc = 2;
    cr = 1/2;  
    P_pattern = [1 1];  
       
elseif mode==18,
    rate = [0 1 1 1];
    N_cbps = 96;
    N_dbps = 72; 
    N_bpsc = 2;
    cr = 3/4;
    P_pattern = [1 1 1 0 0 1 1 1 1 0 0 1 1 1 1 0 0 1];      
    
elseif mode==24,
    rate = [1 0 0 1];
    N_cbps = 192;
    N_dbps = 96; 
    N_bpsc = 4;
    cr = 1/2;
    P_pattern = [1 1]; 
    
elseif mode==36,
    rate = [1 0 1 1];
    N_cbps = 192;
    N_dbps = 144; 
    N_bpsc = 4;
    cr = 3/4;
    P_pattern = [1 1 1 0 0 1 1 1 1 0 0 1 1 1 1 0 0 1]; 
    
elseif mode==48,
    rate = [0 0 0 1];
    N_cbps = 288;
    N_dbps = 192;  
    N_bpsc = 6;
    cr = 2/3;
    P_pattern = [1 1 1 0 1 1 1 0 1 1 1 0];

elseif mode==54,
    rate = [0 0 1 1];
    N_cbps = 288;
    N_dbps = 216;
    N_bpsc = 6;
    cr = 3/4;
    P_pattern = [1 1 1 0 0 1 1 1 1 0 0 1 1 1 1 0 0 1]; 
    
else
    % wrong mode
    
end
    
reserved = 0;

tx_length = zeros(1,N_TX_LENGTH_FIELD);
tx_len_s = dec2bin(tx_len,N_TX_LENGTH_FIELD);
for k=1:N_TX_LENGTH_FIELD,
   tx_length(k) = tx_len_s(N_TX_LENGTH_FIELD-k+1) == '1';
end

parity = mod(sum([rate reserved tx_length]),2);

tail = zeros(1, N_TAIL_FIELD);

tx_data = [rate reserved tx_length parity tail]; 

%fprintf('%i ', tx_data);
%fprintf('\n');

% encoding
tx_data_enc = convenc(tx_data,trellis);

%fprintf('%i ', tx_data_enc);
%fprintf('\n');

Ncbps_sig = 48;
Ndbps_sig = 24;

%s_sig = 1;
%k = 0:1:Ncbps_sig-1;
%perm1 = (Ncbps_sig/16)*mod(k,16) + floor(k/16) + 1;
%perm2 = s_sig*floor(k/s_sig) + mod(k+Ncbps_sig-floor(16*k/Ncbps_sig), s_sig) + 1;
%tx_data_perm(perm1) = tx_data_enc;
%tx_data_perm(perm2) = tx_data_perm;

tx_data_perm = permuter(tx_data_enc, Ncbps_sig, 1);

%fprintf('%i ', tx_data_perm);
%fprintf('\n');

tx_data_symb = 2*tx_data_perm - 1;

tx_freq = zeros(1, N_fft);
tx_freq(P_data_mod) = tx_data_symb;
tx_freq(P_pilots_mod) = [1 1 1 -1];

tx_signal = ifft(tx_freq);
tx_signal = [tx_signal(end-N_gi+1:end) tx_signal tx_signal(1)]; % one sample overlap

tx_signal(1) = tx_signal(1)*0.5;
tx_signal(end) = tx_signal(end)*0.5;

% Generate data symbols
N_sym = ceil((N_SERVICE_FIELD+8*tx_len+N_TAIL_FIELD)/N_dbps);
N_data = N_sym * N_dbps;
N_pad = N_data - (N_SERVICE_FIELD+8*tx_len+N_TAIL_FIELD);

data_service = zeros(1, N_SERVICE_FIELD);
data_tail = zeros(1, N_TAIL_FIELD);
data_pad = zeros(1, N_pad);
data = [data_service data_psdu data_tail data_pad];

data_scr = scrambler(data, ones(1,N_SCRAMBLER_STATES));

data_scr(1+N_SERVICE_FIELD+tx_len*8:N_SERVICE_FIELD+tx_len*8+N_TAIL_FIELD) = data_tail;

% fprintf('%i ', data_scr(N_SERVICE_FIELD+tx_len*8+N_TAIL_FIELD+1:end));
% fprintf('\n');
% pause

data_enc = convenc(data_scr, trellis);

% Puncturing
data_punct = zeros(1, round(length(data_scr)/cr));
P_len = length(P_pattern);
P_nz = length(find(P_pattern == 1));
for k=1:length(data_enc)/P_len,
    data_chunk = data_enc(1+(k-1)*P_len:k*P_len);
    data_chunk = data_chunk(P_pattern==1);
    data_punct(1+(k-1)*P_nz:k*P_nz) = data_chunk;
end

% Symbol allocation, interleaving, maping, OFDM modulation
tx_data = zeros(1, N_sym*(N_fft+N_gi)+1); 
pilot_polarity = scrambler(zeros(1,N_sym+1), ones(1,N_SCRAMBLER_STATES));
for m=1:N_sym,
    
    data_sym = data_punct(1+(m-1)*N_cbps:m*N_cbps);
    data_perm = permuter(data_sym, N_cbps, N_bpsc);
    tx_freq = mapper(data_perm, N_bpsc);
    tx_sym = zeros(1, N_fft);
    tx_sym(P_data_mod) = tx_freq;
    tx_sym(P_pilots_mod) = [1 1 1 -1] * (2*pilot_polarity(m+1)-1);

    tx_sym = ifft(tx_sym);
    tx_sym = [tx_sym(end-N_gi+1:end) tx_sym tx_sym(1)]; %#ok<*AGROW> % one sample overlap

    tx_sym(1) = tx_sym(1)*0.5;
    tx_sym(end) = tx_sym(end)*0.5;
    
    tx_data(1+(m-1)*(N_fft+N_gi):m*(N_fft+N_gi)+1) = ...
        tx_data(1+(m-1)*(N_fft+N_gi):m*(N_fft+N_gi)+1) + tx_sym;   

end

% Generate final burst
N_pg = 128;

tx_out = zeros(1, (N_gi+N_fft)*(3+N_sym) + N_pg); 

% Adding Long training sequence
tx_out(1:2*(N_gi+N_fft)+1) = tx_out(1:2*(N_gi+N_fft)+1) + tx_long;

% Adding Signal symbol
tx_out(2*(N_gi+N_fft)+1:3*(N_gi+N_fft)+1) = ...
    tx_out(2*(N_gi+N_fft)+1:3*(N_gi+N_fft)+1) + tx_signal;

% Adding data symbols
tx_out(3*(N_gi+N_fft)+1:(3+N_sym)*(N_gi+N_fft)+1) = ...
    tx_out(3*(N_gi+N_fft)+1:(3+N_sym)*(N_gi+N_fft)+1) + tx_data;


end

